/* eslint-disable */
const { resolve } = require('path');
var webpack = require('webpack')
const VisualizerPlugin = require('webpack-visualizer-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin')

const dist = resolve(__dirname, '../../beat/editor/js');

const isExternal = (module) => module.context && module.context.indexOf('node_modules') === -1;

module.exports = {
	entry: {
		main: [
			'react-hot-loader/patch',
			// all our code starts here
			'./main.jsx',
		]
	},
	mode: 'development',
	devtool: 'inline-source-map',
	output: {
		// use [name] for sanity checks & debugging
		filename: '[name].js',
		// output in './dist/'
		path: dist,
		publicPath: '/',
		globalObject: 'this',
	},
	module: {
		rules: [
			// load our js files with babel-loader
			{
				test: /\.js(x)?$/,
				loader: 'babel-loader',
				// js deps shouldnt need any loaders
				exclude: /node_modules/
			},
			// web/shared worker files
			{
				test: /\.worker\.js$/,
				use: [
					'worker-loader',
					'babel-loader',
				]
			},
			// load css with:
			{
				test: /\.css$/,
				use: [
					// style-loader -> pulls in the file initially
					'style-loader',
					// css-loader -> handles css
					'css-loader',
					// postcss-loader -> postprocess css (like babel for css)
					'postcss-loader'
				],
				exclude: /node_modules/,
			},
			{
				test: /\.css$/,
				use: [
					// style-loader -> pulls in the file initially
					'style-loader',
					// css-loader -> handles css
					'css-loader',
				],
				include: /node_modules/,
			},
			// files should be handled by file-loader
			{
				test: /\.(png|jpeg(2)?|gif)(\?[a-z0-9]+)?$/,
				loader: 'file-loader'
			},
			// fonts should be handled by file-loader but with special output format
			{
				test: /\.(ttf|otf|eot|woff(2)?)(\?[a-z0-9]+)?$/,
				loader: 'file-loader?name=fonts/[name].[ext]?[hash]'
			},
			// special loader for svg
			{
				test: /\.(svg)(\?[a-z0-9]+)?$/,
				loader: 'svg-inline-loader'
			}
		]
	},
	resolve: {
		extensions: [
			'.js', '.jsx'
		],
		alias: {
			// icons
			/*
			'icon': resolve(__dirname, './node_modules/feather-icons/dist/icons'),
			*/
			'@store': resolve(__dirname, './src/store'),
			'@helpers': resolve(__dirname, './src/helpers'),
			'@test': resolve(__dirname, './test'),
		}
	},
	devServer: {
	},
	plugins: [
		//uses the index.ejs template to generate dynamic html pages based on hashes
		new HtmlWebpackPlugin({
			alwaysWriteToDisk: true,
			template: 'index.html'
		}),


		/*******************
		 * POST-PROCESSING *
		 *******************/

		//generate a little viz for insight into bundle contents/sizes
		new VisualizerPlugin()
	]
}

if(process.env.NODE_ENV === 'debug'){
	// setup the devserver
	module.exports.devServer = {
		// use the history api if necessary
		historyApiFallback: true,

		// serve the content in the output path
		contentBase: dist,

		port: 9101
	}
}

if(process.env.NODE_ENV === 'test'){
	// when testing with karma, almost all plugins are useless
	module.exports.plugins = [
		new webpack.NamedModulesPlugin()
	];
	// make sure to use fancy sourcemaps
	module.devtool = 'inline-source-map'
}

// PROD
// TODO: pull out into different webpack config
if (process.env.NODE_ENV === 'production') {
	module.mode = 'production';
	module.exports.output.publicPath = './';
	module.exports.plugins = [
		// make sure the env is the string 'production'
		new webpack.DefinePlugin({
			'process.env': {
				// need the nested quotes!
				NODE_ENV: '"production"'
			}
		}),
	].concat(module.exports.plugins);
	module.exports.optimization = {
		minimizer: [new TerserPlugin()]
	};
}
